#include once "mdList.bi"
#include once "fbgfx.bi"

namespace GuiState
  const active = 0, deactivated = 1, activated = 2, noBorder = 4, hidden = 128
end namespace

type TGuiButton
  as integer id, x, y, w, h, fontsize, state
  as string title, key, info
  as FB.Image ptr img
  declare constructor
  declare constructor(id as integer, x as integer, y as integer, w as integer, h as integer, title as string, _
                      key as string = "", info as string = "", state as integer = GuiState.active)
  declare constructor(id as integer, imgStr as string, x as integer, y as integer, w as integer, h as integer, _
                      key as string = "", info as string = "", state as integer = GuiState.active)
  declare operator cast as string
  declare sub show(state as integer = GuiState.active)
end type

constructor TGuiButton
end constructor

constructor TGuiButton(id as integer, x as integer, y as integer, w as integer, h as integer, title as string, _
                       key as string = "", info as string = "", state as integer = GuiState.active)
  this.id       = id
  this.x        = x
  this.y        = y
  this.w        = w
  this.h        = h
  this.state    = state
  this.title    = title
  this.key      = key
  this.info     = info
  this.fontsize = 13
end constructor

constructor TGuiButton(id as integer, imgStr as string, x as integer, y as integer, w as integer, h as integer, _
                       key as string = "", info as string = "", state as integer = GuiState.active)
  this.id       = id
  this.x        = x
  this.y        = y
  this.w        = w
  this.h        = h
  this.state    = state
  this.title    = ""
  this.key      = key
  this.info     = info
  this.fontsize = 13
  dim as integer imgW, imgH, filenr = freefile
  if open(imgStr for binary as #filenr) = 0 then
    if lof(filenr) >= 32 then
      get #filenr, 19, imgW
      get #filenr, 23, imgH
      close #filenr
      this.img = imagecreate(imgW, imgH)
      bload imgStr, this.img
    else
      print 1 : getkey
      close #filenr
    end if
  end if
end constructor

operator TGuiButton.cast as string
  return mki(x) & mki(y) & mki(w) & mki(h) & mki(state) & mki(len(title)) & title & mki(len(key)) & key
end operator

operator =(b1 as TGuiButton, b2 as TGuiButton) as integer
  return b1.id = b2.id
end operator

mdListDeclare(TGuiButton)

type TGuiEvent
  as integer id, mButton
  as string key
end type

type TGui
  as mdList(TGuiButton) buttons
  declare destructor
  declare sub show(state as integer = GuiState.active)
  declare function getEvent(key as string = "") as TGuiEvent
  declare function getButton(id as integer) as TGuiButton
  declare sub setButton(button as TGuiButton)
end type

destructor TGui
  dim as TGuiButton b
  foreach(TGuiButton, b, buttons)
    if b.img then imagedestroy b.img
  nexteach
  buttons.clear
end destructor

sub TGuiButton.show(stateVar as integer = GuiState.active)
  if state and GuiState.hidden then exit sub
  dim as uinteger col = iif(state = GuiState.deactivated or stateVar = GuiState.deactivated, &h999999, 0)
  if (state and GuiState.noBorder) = 0 then line (x, y)-step(w, h), col, bf
  if (state and GuiState.activated) = 0 then line (x+2, y+2)-step(w-4, h-4), rgb(255, 255, 255), bf
  if img then
    dim as integer x_, y_, w_, h_
    if w > img->width + 4 then
      w_ = img->width
      x_ = (w-img->width)\2
    else
      w_ = w - 4
      x_ = 2
    end if
    if h > img->height + 4 then
      h_ = img->height
      y_ = (h-img->height)\2
    else
      h_ = h - 4
      y_ = 2
    end if
    if state and GuiState.activated then
      put (x+x_, y+y_), img, ((img->width-w_)\2, (img->height-h_)\2)-step(w_-1, h_-1), preset
    else
      put (x+x_, y+y_), img, ((img->width-w_)\2, (img->height-h_)\2)-step(w_-1, h_-1), pset
    end if
  else
    dim as FB.Image ptr tempImg = imageCreate(w-4, h-4, rgb(255, 255, 255))
    centerFT tempImg, title, font, fontsize, col
    if state and GuiState.activated then
      put (x+2, y+2), tempImg, preset
    else
      put (x+2, y+2), tempImg, pset
    end if
    imagedestroy tempImg
  end if
  if len(info) then
    dim as TCoord coord = FTsize(info, font, 12)
    line (x+w\2-coord.x\2-1, y+h+15-coord.y\2-1)-step(coord.x+2, coord.y+2), &hffffff, bf 
    centerFT x+w\2, y+h+15, info, font, 12, col
  end if
end sub

function TGui.getEvent(key as string = "") as TGuiEvent
  dim as TGuiEvent ret
  dim as TGuiButton b
  dim as integer mx, my, mb, s
  getmouse mx, my,, mb
  foreach(TGuiButton, b, buttons)
    s = b.state and not (GuiState.activated or GuiState.noBorder)
    if s = GuiState.active then
      if key <> "" andalso key = b.key then
        ret.id = b.id
        ret.mButton = -1
        ret.key = key
        exit while
      end if
      if mb > 0 andalso checkMouseIn(b.x, b.y, b.w, b.h) <> 0 then
        ret.id = b.id
        ret.mButton = mb
        ret.key = ""
        exit while
      end if
    end if
  nexteach
  return ret
end function

function TGui.getButton(id as integer) as TGuiButton
  dim as TGuiButton b
  foreach(TGuiButton, b, buttons)
    if b.id = id then return b
  nexteach
end function

sub TGui.setButton(button as TGuiButton)
  dim as TGuiButton b
  dim as integer i = 0
  foreach(TGuiButton, b, buttons)
    if b.id = button.id then
      buttons.add(i, button)
      buttons.remove(i+1)
      exit sub
    end if
    i += 1
  nexteach
  buttons.add(button)
end sub

sub TGui.show(state as integer = GuiState.active)
  dim as TGuiButton b
  foreach(TGuiButton, b, buttons)
    b.show state
  nexteach
end sub
