#include once "TWord.bi"
mdMapDeclare(long, TWord)
mdListDeclare(long)
const ENDOFLIST = !"\0"

type TListPoint
  as ulong  id, special, addition
  as single factor
  as double position
  declare constructor
  declare constructor(i as long, p as long = 0, a as long = 0)
  declare operator cast as string
  declare sub modify(lvl as integer)
end type

constructor TListPoint
end constructor

constructor TListPoint(i as long, p as long = 0, a as long = 0)
  id       = i
  position = p
  addition = a
end constructor

operator =(l1 as TListPoint, l2 as TListPoint) as integer
  return l1.id = l2.id andalso l1.special = l2.special
end operator

operator TListPoint.cast as string
  return mkl(id) & mkl(position) & mkl(addition)
end operator

sub TListPoint.modify(lvl as integer)
  ' modify this.addition and this.factor depending on word difficulty
  if factor = 0 then factor = 2.5
  factor += 0.1-(5-lvl)*(0.08+(5-lvl)*0.02)
  if factor < 1.3 then factor = 1.3
  if lvl < 3 or addition < 1 then
    addition = 1
  elseif addition < 6 then
    addition = 6
  else
    addition *= factor
  end if
  ' After each repetition assess the quality of repetition response in 0-5 grade scale:
  ' 5 - perfect response
  ' 4 - correct response after a hesitation
  ' 3 - correct response recalled with serious difficulty
  ' 2 - incorrect response; where the correct one seemed easy to recall
  ' 1 - incorrect response; the correct one remembered
  ' 0 - complete blackout.
  ' algorithm from https://www.supermemo.com/english/ol/sm2.htm
end sub

mdListDeclare(TListPoint)

type TList
  as double lastSave
  as integer size, notChecked
  as string listname
  as mdList(TListPoint) list
  declare constructor
  declare constructor(s as string)
  declare operator let(l as TList)
  declare sub add cdecl(lst as TList = TList, lvlMin as integer, lvlMax as integer, ...)
  declare sub del cdecl(lvlMin as integer, lvlMax as integer, ...)
  declare sub mix
  declare sub load(byval ln as string = "")
  declare sub save(byval ln as string = "")
  declare sub sort
  declare sub update
end type

constructor TList
end constructor

constructor TList(s as string)
  listname = s
end constructor

operator TList.let(l as TList)
  lastSave = l.lastSave
  size     = l.size
  notChecked  = l.notChecked
  listname = l.listname
  list.clear
  list.addAll(l.list)
end operator

dim shared as mdMap(long, TWord) globallist

sub TList.add cdecl(wordlist as TList = TList, lvlMin as integer, lvlMax as integer, ...)
  ' Adds elements of wordlist.list to this.list, if all given tags (va_arg list) correspond.
  ' If wordlist.list is empty, words are taken from the global wordlist.
  dim as any ptr argument = va_first
  dim as string k, v
  dim entry as TListPoint, w as TWord, tag as mdMap(string, string)

  if wordlist.list.isEmpty then
    dim as mdList(long) ks = globallist.keyset
    dim as long i
    foreach(long, i, ks)
      wordlist.list.add(i)
    nexteach
  end if

  do
    k = *va_arg(argument, zstring ptr)
    if k = ENDOFLIST then exit do
    argument = va_next(argument, zstring ptr)
    v = *va_arg(argument, zstring ptr)
    if v = ENDOFLIST then
      tag.put(k, "")
      exit do
    end if
    tag.put(k, v)
    argument = va_next(argument, zstring ptr)
  loop
  dim as mdSet(string) set = tag.keyset
 
  foreach(TListPoint, entry, wordlist.list)
    w = globallist.get(entry.id)
    if w.lession >= lvlMin and w.lession <= lvlMax then
      foreach(string, k, set)
        select case k
          case "word"
            if w.word <> v then continue while, while      ' means: continue foreach, foreach
          case "mark"
            if w.mark <> cint(v) then continue while, while
          case "trans"
            if not w.translate.contains(v) then continue while, while
          case "group"
            if w.groupname <> v then continue while, while
          case "tag"
            if not w.tag.contains(v) then continue while, while
          case else
            if w.map.containsKey(k) = 0 then continue while, while
            if v <> "" andalso w.map.get(k) <> v then continue while, while
        end select
      nexteach
      list.add(entry)
    end if
  nexteach
end sub

sub TList.del cdecl(lvlMin as integer, lvlMax as integer, ...)
  ' delete elements of this.list, if all given tags (va_arg list) correspond
  dim as any ptr argument = va_first
  dim as string k, v
  dim entry as TListPoint, w as TWord, tag as mdMap(string, string)

  do
    k = *va_arg(argument, zstring ptr)
    if k = ENDOFLIST then exit do
    argument = va_next(argument, zstring ptr)
    v = *va_arg(argument, zstring ptr)
    if v = ENDOFLIST then
      tag.put(k, "")
      exit do
    end if
    tag.put(k, v)
    argument = va_next(argument, zstring ptr)
  loop
  dim as string keys()
  tag.keySet.toArray(keys())
  dim as mdSet(string) set = tag.keyset
 
  foreach(TListPoint, entry, list)
    w = globallist.get(entry.id)
    if w.lession >= lvlMin and w.lession <= lvlMax then
      foreach(string, k, set)
        select case k
          case "word"
            if w.word <> v then continue while, while      ' means: exit foreach, foreach
          case "mark"
            if w.mark <> cint(v) then continue while, while
          case "trans"
            if not w.translate.contains(v) then continue while, while
          case "group"
            if w.groupname <> v then continue while, while
          case "tag"
            if not w.tag.contains(v) then continue while, while
          case else
            if w.map.containsKey(k) = 0 then continue while, while
            if v <> "" andalso w.map.get(k) <> v then continue while, while
        end select
      nexteach
      list.remove(entry)
    end if
  nexteach
end sub

sub TList.mix
  ' mix elements in the list (regardless of position and addition)
  dim listPoint as TListPoint, index as integer, i as integer = 0
  foreach(TListPoint, listPoint, list)
    if listPoint.position > 0 then exit while
    i += 1
  nexteach
  for k as integer = i-1 to 0 step -1
    index = int(rnd*k)
    listPoint = list.get(index)
    list.remove(index)
    if k = size-1 then list.add(listPoint) else list.add(k, listPoint)
  next
end sub

sub TList.load(byval ln as string = "")
  dim as integer filenumber = freefile
  dim as TListPoint listPoint
  if ln = "" then ln = listname
  open exepath & "/" & LANGUAGE & "/" & ln & ".lst" for binary as #filenumber
  get #filenumber,, lastSave
  do until eof(filenumber)
    get #filenumber,, listPoint
    list.remove(listPoint)  ' remove from the current meaningless position ...
    list.add(listPoint)     ' ... and add to the correct position at the end
  loop
  close #filenumber
  update
end sub

sub TList.save(byval ln as string = "")
  dim as TListPoint listPoint
  dim as integer filenumber = freefile
  if ln = "" then ln = listname
  open exepath & "/" & language & "/" & ln & ".lst" for binary as #filenumber
  put #filenumber,, lastSave
  foreach(TListPoint, listPoint, list)
    put #filenumber,, listPoint
  nexteach
  close #filenumber
end sub

sub TList.sort
  ' sort by position (if something got wrong)
  dim as TListPoint cur, last = list.get(0)
  dim as integer i, index = 1
  do
    cur = list.get(index)
    if cur.position < last.position then
      for i = index-2 to 0 step -1
        if list.get(i).position <= cur.position then exit for
      next
      list.remove(index)
      list.add(i+1, cur)
      last = cur
      index = i+2
    else
      last = cur
      index += 1
    end if
  loop until index > size
end sub

sub TList.update
  dim as TListPoint listPoint
  size = list.size
  notChecked = 0
  foreach(TListPoint, listPoint, list)
    if listPoint.position <= now then notChecked += 1 else exit while
  nexteach
end sub

